#!/usr/bin/env python3
from nautilus_trader.backtest.node import BacktestDataConfig
from nautilus_trader.backtest.node import BacktestEngineConfig
from nautilus_trader.backtest.node import BacktestNode
from nautilus_trader.backtest.node import BacktestRunConfig
from nautilus_trader.backtest.node import BacktestVenueConfig
from nautilus_trader.config import ImportableStrategyConfig
from nautilus_trader.config import LoggingConfig
from nautilus_trader.data.config import DataEngineConfig
from nautilus_trader.model import Quantity, BarType
from nautilus_trader.model import QuoteTick
from nautilus_trader.persistence.catalog import ParquetDataCatalog
from nautilus_trader.core.rust.model import TriggerType, TimeInForce
from nautilus_trader.model import TradeTick, Bar
from nautilus_trader.model import InstrumentId
from nautilus_trader.core.message import Event
from nautilus_trader.indicators.macd import MovingAverageConvergenceDivergence
from nautilus_trader.model import InstrumentId
from nautilus_trader.model import Position
from nautilus_trader.model.enums import OrderSide
from nautilus_trader.model.enums import PositionSide
from nautilus_trader.model.enums import PriceType
from nautilus_trader.core.rust.common import LogColor
from nautilus_trader.model.events import PositionOpened, PositionChanged, OrderAccepted, OrderFilled
from nautilus_trader.trading.strategy import Strategy
from nautilus_trader.trading.strategy import StrategyConfig

from decimal import Decimal

from pandas import Timedelta
from datetime import datetime
from math import ceil

catalog = ParquetDataCatalog("/home/p/p-dev/nautilus/catalog")
instrument_id = InstrumentId.from_str("APTUSDT-PERP.BINANCE")

venue = BacktestVenueConfig(
    name="BINANCE",
    oms_type="NETTING",
    account_type="MARGIN",
    base_currency="USDT",
    starting_balances=["100 USDT"],
    default_leverage=5,
    leverages={"APTUSDT-PERP.BINANCE": 5}
)

data = BacktestDataConfig(
    catalog_path=str(catalog.path),
    data_cls=TradeTick,
    instrument_ids=[instrument_id],
    start_time="2024-12-01 00:00:00",
    end_time="2024-12-01 00:00:30"
)





class BarStrategyConfig(StrategyConfig, frozen=True):
    instrument_id: InstrumentId
    aggregation_period: str
    aggregation_period_seconds: int


class BarStrategy(Strategy):
    def __init__(self, config: BarStrategyConfig):
        super().__init__(config=config)
        self.log.info("Initialization of MACDStrategy")

        # Convenience
        self.position: Position | None = None
        self.bar_type = BarType.from_str(f"{self.config.instrument_id}-{self.config.aggregation_period}-LAST-INTERNAL")

        self.bar_reset = True
        self.bar_open = Decimal(0)
        self.bar_high = Decimal(0)
        self.bar_low = Decimal(0)
        self.bar_close = Decimal(0)

    def ceil_current_time(self) -> datetime:
        time: datetime = self.clock.utc_now()

        ret = (
                time.replace(second=0, microsecond=0) + Timedelta(seconds=ceil(time.second/self.config.aggregation_period_seconds) * self.config.aggregation_period_seconds)
        )

        if time.microsecond > 0:
            ret += Timedelta(seconds=self.config.aggregation_period_seconds)

        return ret


    def on_start(self):
        self.log.info("Start of Strategy")

        start_time = self.ceil_current_time()
        self.log.info(f"First bar_reset will occur at {start_time}.")

        self.clock.set_timer("_on_second", Timedelta(seconds=self.config.aggregation_period_seconds), start_time=start_time - Timedelta(seconds=self.config.aggregation_period_seconds), stop_time=None,
                             callback=self.__on_second)

        self.subscribe_trade_ticks(instrument_id=self.config.instrument_id)
        self.subscribe_bars(self.bar_type)




    def __on_second(self, event):
        bar = self.received_bar

        self.log.info(f"Received bar {bar}")
        self.log.info(f"Expected bar open={self.bar_open}, high={self.bar_high}, low={self.bar_low}, close={self.bar_close}")
        assert bar.low == self.bar_low
        assert bar.high == self.bar_high
        assert bar.open == self.bar_open
        assert bar.close == self.bar_close

        self.bar_reset = True
        self.bar_open = self.bar_close
        self.bar_high = self.bar_close
        self.bar_low = self.bar_close


    def on_stop(self):
        self.log.info("Stop of Strategy")
        self.close_all_positions(self.config.instrument_id)
        self.unsubscribe_trade_ticks(instrument_id=self.config.instrument_id)



    def on_bar(self, bar: Bar):
        self.received_bar = bar




    def on_trade_tick(self, tick: TradeTick):
        if self.bar_reset:
            self.bar_reset = False
            self.bar_open = tick.price
            self.bar_high = tick.price
            self.bar_low = tick.price

        else:
           if self.bar_low > tick.price:
               self.bar_low = tick.price

           if self.bar_high < tick.price:
               self.bar_high = tick.price

        self.bar_close = tick.price


    def on_dispose(self):
        pass  # Do nothing else


engine = BacktestEngineConfig(
    strategies=[
        ImportableStrategyConfig(
            strategy_path="__main__:BarStrategy",
            config_path="__main__:BarStrategyConfig",
            config={
                "instrument_id": instrument_id,
                "aggregation_period": "2-SECOND",
                "aggregation_period_seconds": 2
            },
        )
    ],
    logging=LoggingConfig(log_level="DEBUG"),
    data_engine=DataEngineConfig(
        time_bars_skip_first_non_full_bar=True
    )
)

config = BacktestRunConfig(
    engine=engine,
    venues=[venue],
    data=[data],
    chunk_size=1024*32
)

from nautilus_trader.backtest.results import BacktestResult


node = BacktestNode(configs=[config])

results: list[BacktestResult] = node.run()
